unit UGoal2DStorage;

// Players with load will drop it here when touch and the amount of load stored
// will be your score.

interface

uses UGoal, UPlayer2D;


type TGoal2DStorage = class(TGoal)
  private
    p: TPlayer2D;
    // Position and size of the storage facility:
    X, Y, Size: integer;
    // Capacity, how much load it can store:
    MaxCapacity: double;
    // Actual amount of load in the storage:
    Used: double;
  protected
    procedure ReadQuantity(v: TObject); override;
  public
    constructor create(Sender: TObject; vx, vy, vsize, vmax: integer);
    procedure DrawOnFrame; override;
  end;


implementation

uses UTrack2D, Graphics, ExtCtrls, Math;


constructor TGoal2DStorage.create(Sender: TObject; vx, vy, vsize, vmax: integer);
begin
  inherited create(Sender);
  DisplayName:= 'Store drops';
  // Only gain feelings:
  ReactionTime:= 10;
  Urgency:= 0.5;
  //
  X:= vx;
  Y:= vy;
  MaxCapacity:= vmax;
  Size:= Max(1, vsize);
  p:= nil;
end;


procedure TGoal2DStorage.ReadQuantity(v: TObject);
var
  NewUsed: double;
  Gain: double;
begin
  inherited;
  // By default, no loose or gain:
  OldQuantity:= 10; // 10% is the scale, store 10% of energy for a gain of 1.
  Quantity:=    10;
  // Let see if it store some energy on the box:
  p:= TPLayer2D(v);
  if p.Dead then exit;
  // When you touch it your energy is downloaded into the container:
  if (Used < MaxCapacity) and
     (abs(p.PosX-X)<(p.Size+Size)) and
     (abs(p.PosY-Y)<(p.Size+Size)) then begin
    // How much of the load can I store?
    NewUsed:= Min(MaxCapacity, Used + Max(0, p.Energy-25));
    Gain:= NewUsed - Used;
    // Energy loaded is you gain:
    Quantity:= Quantity + Gain;
    // If it is real, pass the load from player to store:
    if not p.Imagining then begin
      p.Energy:= p.Energy - Gain;
      Used:= NewUsed;
      // Automatically empty the box when it reach 90% full:
      if Used > 0.9 * MaxCapacity then
        Used:= 0;
    end;
  end;
end;

procedure TGoal2dStorage.DrawOnFrame;
var
  C: TCanvas;
begin
  if not Assigned(p) then exit;
  C:= TTrack2D(p.Track).Img.Canvas;
  // A square half filled will do as a storage:
  C.Pen.Color:= clBlue;
  C.Brush.Style:= bsClear;
  C.Rectangle(round(X-Size), round(Y-Size), round(X+Size), round(Y+Size));
  // How full is it?
  C.Pen.Color:= clBlack;
  C.Brush.Color:= clAqua;
  C.Rectangle(round(X-Size), round(Y+Size - ceil(2*Size*(Used/MaxCapacity))), round(X+Size), round(Y+Size));
end;

end.
