unit UGoal2DSweets;

// A lot of energy drops that jump into the player's mouth when touched. The more
// you eat, the more it score, so when you are full it won't score anymore.
// Restricted for Rockets only to simulate evolution: Karts eat rockets...

interface

uses UGoal, UPlayer2D;


type TGoal2DSweets = class(TGoal)
  private
    FirstRun: boolean;
    X:  array [0..99] of integer;
    Y:  array [0..99] of integer;
    SR: array [0..99] of integer; // Sweet points scores (R-eal one)
    SI: array [0..99] of integer; // Points copy to use while I-maginig
    fLeft: integer;
    Continuous: boolean;
    Player: TPlayer2D;
    procedure AddOneDrop(i: integer);
    procedure AddAllDrops;
    procedure SetDropSize(n, v: integer);
    function  GetDropSize(n: integer): integer;
  protected
    procedure ReadQuantity(v: TObject); override;
    property  DropSize[n: integer]: integer read GetDropSize write SetDropSize;
    property  Left: integer read fLeft write fLeft;
  public
    constructor create(Sender: TObject); override;
    procedure Reset; override;
    procedure DrawOnFrame; override;
  end;

const
  NDrops: integer =    25;
  MinDrops: integer =   5; // If only this number of drops remain, create more.


implementation

uses UTrack2D, UTrack, URocket, Graphics, ExtCtrls, Math;


constructor TGoal2DSweets.create(Sender: TObject);
begin
  inherited;
  DisplayName:= 'Get drops';
  // This goal doesn't feel, just add energy, so adjust energy goal instead!
  NominalQuantity:=     1; // Dropd -> x5 happy (5 = FaceValue)
  //ReactionMSeconds:= 5000; // No gain or loose feeling
  //
  FirstRun:= true;
  Player:= Nil;
  Continuous:= false;
end;

procedure TGoal2DSweets.ReadQuantity(v: TObject);
var
  i: integer;
const
  // How much energy a size 1 drop contains:
  FaceValue: integer = 5;
begin
  inherited;
  // Locate the player:
  Player:= TPLayer2D(v);
  // Add lots of sweet drops on the track if not already done:
  if FirstRun then
    AddAllDrops;
  Quantity:= 1;
  OldQuantity:= 1;
  // Dead players don't eat!
  if Player.dead then exit;
  if TTrack(Player.Track).Evolution and not (Player is TRocket) then exit;
  // Did it stepped into a drop?
  for i:= 0 to NDrops-1 do begin
    // Drop is not depleted and player touch it?
    if (DropSize[i] > 0) and
       (abs(Player.PosX-X[i]) < DropSize[i] + Player.Size) and
       (abs(Player.PosY-Y[i]) < DropSize[i] + Player.Size) then begin
      // Touching a drop is enjoyable!
      Quantity:= FaceValue; // Gain feeling of a 10%
      // Drops gives energy, so if you are not full, your energy will increase
      Player.Energy:= Player.Energy + FaceValue;
      // Drop loose one unit:
      DropSize[i]:= DropSize[i]-1;
      // Replace empty drops?
      if (DropSize[i]=0) then begin
        if Continuous then
          AddOneDrop(i)
        else
          if not Player.Imagining then
            dec(fLeft);
      end;
      // No more drops? Create more!
      if (Left <= MinDrops) then
        FirstRun:= true;
    end;
  end;
end;

procedure TGoal2DSweets.SetDropSize(n, v: integer);
begin
  if Player.Imagining then
    SI[n]:= v
  else
    SR[n]:= v;
end;
function  TGoal2DSweets.GetDropSize(n: integer): integer;
begin
  if Player.Imagining then
    result:= SI[n]
  else
    result:= SR[n];
end;


procedure TGoal2DSweets.AddOneDrop(i: integer);
var
  Img: TImage;
  p: TPlayer2D;
  IsInside: boolean;
begin
  // Only redefine empty drops:
  if (SR[i]>0) then exit;
  p:= TPLayer2D(Player);
  Img:= TTrack2D(p.Track).Img;
  // Define size of the drop:
  SR[i]:= 2+round(random*4);
  SI[i]:= SR[i];
  // Define location of the drop:
  repeat
    X[i]:= round((0.1 + random*0.8)*Img.Width);
    Y[i]:= round((0.1 + random*0.8)*Img.Height);
    IsInside:= TTrack2D(p.Track).IsInsideTrack(X[i], Y[i]) and
             TTrack2D(p.Track).IsInsideTrack(round(X[i]+p.Size*1.5), Y[i]) and
             TTrack2D(p.Track).IsInsideTrack(round(X[i]-p.Size*1.5), Y[i]) and
             TTrack2D(p.Track).IsInsideTrack(X[i], round(Y[i]+p.Size*1.5)) and
             TTrack2D(p.Track).IsInsideTrack(X[i], round(Y[i]-p.Size*1.5));
  until IsInside;
end;

procedure TGoal2DSweets.AddAllDrops;
var i: integer;
begin
  for i:= 0 to NDrops-1 do
    AddOneDrop(i);
  FirstRun:= false;
  Left:= NDrops; // If it reach MinDrops it will add more
end;

procedure TGoal2dSweets.DrawOnFrame;
var
  C: TCanvas;
  i, r: integer;
begin
  if FirstRun or (not Assigned(player)) then exit;
  if Player.Imagining then exit;
  C:= TTrack2D(Player.Track).Img.Canvas;
  C.Brush.Color:= clCream;
  C.Pen.Color:= clGreen;
  for i:= 0 to NDrops-1 do begin
    if (SR[i] > 0) then begin
      r:= (SR[i]+1);
      C.Ellipse(round(X[i]-r), round(Y[i]-r), round(X[i]+r), round(Y[i]+r));
    end;
  end;
end;

// Update the imaginary copy of the scores:
procedure TGoal2dSweets.Reset;
var i: integer;
begin
  for i:= 0 to High(SR) do
    SI[i]:= SR[i];
end;


end.
