unit UGoalMouse;

interface

uses UGoal, UPlayer2D, UTeam;

type TGoalMouse = class(TGoal)
  private
    Repel: boolean;
    Player: TPlayer2D;
  protected
    procedure ReadQuantity(v: TObject); override;
  public
    constructor create(Sender: TObject; v: boolean = false);
    procedure DrawOnFrame; override;
  end;

implementation

uses UTrack2D, Graphics, ExtCtrls, Math;


constructor TGoalMouse.create(Sender: TObject; v: boolean = false);
begin
  inherited create(Sender);
  Repel:= v;
  if Repel then
    DisplayName:= 'Avoid mouse'
  else
    DisplayName:= 'Follow mouse';
  NominalQuantity:= 1; // Quantity will directly be a mood coef
end;

procedure TGoalMouse.ReadQuantity(v: TObject);
var
  x, y: integer;
  d: double;
  t: TTrack2D;
begin
  inherited;
  Player:= TPLayer2D(v);
  if Player.Dead then exit;
  t:= TTrack2D(Player.Track);
  if t.MouseIn then begin
    x:= t.MouseX;
    y:= t.MouseY;
    d:= Sqrt(Sqr(Player.PosX-x) + Sqr(Player.PosY-y)) / (1+Player.Size);
    if Repel then begin
      Quantity:= d;
    end else begin
      if (d>0) then
        Quantity:= 1/d
      else
        Quantity:= 100;
    end;
  end;
end;

procedure TGoalMouse.DrawOnFrame;
var
  t: TTrack2D;
  C: TCanvas;
const
  Size: integer = 10;
begin
  if not Assigned(Player) then exit;
  t:= TTrack2D(Player.Track);
  if t.MouseIn then begin
    C:= t.Img.Canvas;
    // A square half filled will do as a storage:
    C.Pen.Color:= clBlue;
    C.Brush.Style:= bsClear;
    C.Ellipse(round(t.MouseX-Size), round(t.MouseY-Size), round(t.MouseX+Size), round(t.MouseY+Size));
    C.MoveTo(round(t.MouseX), round(t.MouseY-Size));
    C.LineTo(round(t.MouseX), round(t.MouseY+Size));
    C.MoveTo(round(t.MouseX-Size), round(t.MouseY));
    C.LineTo(round(t.MouseX+Size), round(t.MouseY));
  end;
end;

end.
