unit UPolygon2D;

interface

uses
  ExtCtrls, Graphics, Math, Types, DateUtils, SysUtils, Contnrs, Forms,
  UTrack2D, UState, UPlayer, UPlayer2D, UPhysic2D, UUtils;


type
  TPolygon2D = class(TPhysic2D)
  private
    fNumSides: integer;
    procedure SetNumSides(n: integer);
  protected
    procedure CalcNodes; override;
  public
    constructor create(x, y, a: double; n: integer);
    property NumSides: integer read fNumSides write SetNumSides;
  end;


implementation


constructor TPolygon2D.create(x, y, a: double; n: integer);
const
  MaxRot: double = 4;
begin
  inherited create(x, y, a);
  NumSides:= n;
  Density:= 2; // Heavier than rockets
  Uncertainty:= 5; // in m/s2/s (velocity change per second)
  VelA:= (0.5 - random) * MaxRot;
end;

procedure TPolygon2D.SetNumSides(n: integer);
begin
  fNumSides:= Max(3, n);
  NodesNeedIni:= true;
end;


// *********************
// **  DRAWING STUFF  **
// *********************

procedure TPolygon2D.CalcNodes;
var
 i: integer;
 a: double;
begin
  // Nodes is a dynamic array, so I must set size on first use.
  if NodesNeedIni then begin
    SetLength(Nodes, NumSides);
    SetLength(NodesR, NumSides);
    NodesNeedIni:= false;
    NodesNeedRecalc:= true;
  end;
  // If nothing changed from last time, don't waste time:
  if NodesNeedRecalc then begin
    for i:= Low(Nodes) to High(Nodes) do begin
      a:= Angle + 2*PI/NumSides*i;
      NodesR[i, 1]:= PosX + Size*cos(a);
      NodesR[i, 2]:= PosY + Size*sin(a);
      Nodes[i].X:= round(NodesR[i,1]);
      Nodes[i].Y:= round(NodesR[i,2]);
    end;
    NodesNeedRecalc:= false;
    VectorsNeedRecalc:= true;
  end;
end;

end.
