unit UThing2D;

// Physic 2D objet without any intelligence: A stone, a ball, a falling bomb...

interface

uses
  ExtCtrls, Graphics, Math, Types, DateUtils, SysUtils, Contnrs, Forms,
  UTrack2D, UState, UPlayer, UPlayer2D, UUtils;

type
  TThing2D = class(TPlayer2D)
  private
   fNumSides: integer;
   procedure SetNumSides(n: integer);
  protected
    procedure CalcNodes; override;
    // Simulation code:
    procedure InSimulation; override;
  public
    constructor create(x, y, a: double; n: integer);
    property NumSides: integer read fNumSides write SetNumSides;
  end;


implementation

uses UTrack;


// ****************************
// **  ADMINISTRATIVE STUFF  **
// ****************************

constructor TThing2D.create(x, y, a: double; n: integer);
begin
  inherited create;
  PlaceHere(x, y, a);
  NumSides:= n;
  Size:= 8;
  Density:= 2; // A rock!
  Inanimated:= true;
  SetIniState;
end;


// *********************
// **  DRAWING STUFF  **
// *********************

procedure TThing2D.SetNumSides(n: integer);
begin
  fNumSides:= Max(3, n);
  NodesNeedIni:= true;
end;

procedure TThing2D.CalcNodes;
var
 i: integer;
 a: double;
const
  Angles: array[0..2] of double = (0.0, 2/3, 4/3);
begin
  // Nodes is a dynamic array, so I must set size on first use.
  if NodesNeedIni then begin
    SetLength(Nodes, NumSides);
    SetLength(NodesR, NumSides);
    NodesNeedIni:= false;
    NodesNeedRecalc:= true;
  end;
  // If nothing changed from last time, don't waste time:
  if NodesNeedRecalc then begin
    for i:= Low(Nodes) to High(Nodes) do begin
      a:= Angle + 2*PI/NumSides*i;
      NodesR[i, 1]:= PosX + Size*cos(a);
      NodesR[i, 2]:= PosY + Size*sin(a);
      Nodes[i].X:= round(NodesR[i,1]);
      Nodes[i].Y:= round(NodesR[i,2]);
    end;
    NodesNeedRecalc:= false;
    VectorsNeedRecalc:= true;
  end;
end;


// ***********************
// **  SIMULATION STUFF **
// ***********************

// Let run the simulation for a delta time and update state acordingly:
procedure TThing2D.InSimulation;
var
  s: double;
const
  AccY: double = 9.8; // g
begin
  // Inherited will decide if I am dead, out of track, etc.:
  inherited;
  s:= SimSeconds;
  VelX:= VelX * (1-0.01*s);
  VelY:= VelY * (1-0.01*s) + AccY * s;
  VelRot:= VelRot * (1-0.01*s);
  // Remember: Pos(t+s) = Pos(t) + Vel(t)*s + 1/2 Acc(t)*s^2
  Angle:=  Angle + VelRot * s;
  PosX:= PosX + VelX * s;
  PosY:= PosY + VelY * s + AccY * Sqr(s)/2;
  // Update nodes of the Thing2D skeleton:
  NodesNeedRecalc:= true;
end;


end.
